@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2017 gnome-pomodoro contributors
+ * Copyright (c) 2011-2021 gnome-pomodoro contributors
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,9 +20,8 @@
 
 const Signals = imports.signals;
 
-const { Atk, Clutter, GLib, GObject, Meta, St, Pango } = imports.gi;
+const { Atk, Clutter, GLib, GObject, Meta, Shell, St, Pango } = imports.gi;
 
-const GrabHelper = imports.ui.grabHelper;
 const Layout = imports.ui.layout;
 const Lightbox = imports.ui.lightbox;
 const Main = imports.ui.main;
@@ -54,35 +53,15 @@ const IDLE_TIME_TO_OPEN = 60000;
 const IDLE_TIME_TO_CLOSE = 600;
 const MIN_DISPLAY_TIME = 500;
 
-const FADE_IN_TIME = 200;
-const FADE_IN_OPACITY = 0.45;
+const FADE_IN_TIME = 300;
+const FADE_OUT_TIME = 300;
 
-const FADE_OUT_TIME = 200;
+const BLUR_BRIGHTNESS = 0.4;
+const BLUR_SIGMA = 20.0;
 
 const OPEN_WHEN_IDLE_MIN_REMAINING_TIME = 3.0;
 
-// A single pass blur approximation, with hacky brightness param
-const BLUR_FRAGMENT_SHADER = '\
-uniform sampler2D tex; \
-uniform float x_step, y_step, factor, brightness; \
-\
-const float[7] weights = float[7](0.1452444, 0.1314128, 0.1115940, 0.0831466, 0.0543562,  0.0311780,  0.0156902); \
-const float[7] offsets = float[7](0.0000000, 2.4876390, 4.4711854, 6.4547903, 8.4384988, 10.4223371, 12.4063545); \
-\
-void main () { \
-  vec2 uv = vec2(cogl_tex_coord_in[0].st); \
-  vec2 pixel_size = vec2(x_step * factor, y_step * factor); \
-  vec4 color = vec4(0.0); \
-  color += weights[0] * texture2D(tex, uv); \
-  for (int i=1; i < 7; i++) { \
-    color += \
-      weights[i] * texture2D(tex, uv - pixel_size * offsets[i]) + \
-      weights[i] * texture2D(tex, uv + pixel_size * offsets[i]); \
-  } \
-  color.rgb *= 1.0 - (1.0 - sqrt(brightness)) * factor; \
-  cogl_color_out = color; \
-}';
-
+const DEFAULT_BACKGROUND_COLOR = Clutter.Color.from_pixel(0x000000ff);
 
 var State = {
     OPENED: 0,
@@ -92,197 +71,110 @@ var State = {
 };
 
 
-var BlurEffect = GObject.registerClass({
-    Properties: {
-        'orientation': GObject.ParamSpec.enum(
-            'orientation', 'orientation', 'orientation',
-            GObject.ParamFlags.READWRITE,
-            Clutter.Orientation, Clutter.Orientation.HORIZONTAL
-        ),
-        'brightness': GObject.ParamSpec.float(
-            'brightness', 'brightness', 'brightness',
-            GObject.ParamFlags.READWRITE,
-            0, 1, 1
-        ),
-        'factor': GObject.ParamSpec.float(
-            'factor', 'factor', 'factor',
-            GObject.ParamFlags.READWRITE,
-            0, 1, 0
-        ),
-    },
-}, class PomodoroBlurEffect extends Clutter.ShaderEffect {
-    _init(params) {
+var BlurredLightbox = GObject.registerClass(
+class PomodoroBlurredLightbox extends Lightbox.Lightbox {
+    _init(container, params) {
         params = Params.parse(params, {
-            orientation: Clutter.Orientation.HORIZONTAL,
-            brightness: 1.0,
-            factor: 0.0,
+            inhibitEvents: false,
+            width: null,
+            height: null,
         });
 
-        this._orientation = undefined;
-        this._brightness = undefined;
-        this._factor = undefined;
-
-        super._init({ shader_type: Clutter.ShaderType.FRAGMENT_SHADER });
-
-        this.set_shader_source(BLUR_FRAGMENT_SHADER);
-
-        this.orientation = params.orientation;
-        this.brightness = params.brightness;
-        this.factor = params.factor;
-    }
-
-    get brightness() {
-        return this._brightness;
-    }
-
-    set brightness(value) {
-        if (this._brightness == value)
-            return;
-        this._brightness = value;
-        this.set_uniform_value('brightness', GObject.Float(this._brightness));
-        this.notify('brightness');
-    }
-
-    get factor() {
-        return this._factor;
-    }
-
-    set factor(value) {
-        if (this._factor == value)
-            return;
-        this._factor = value;
-        this.set_uniform_value('factor', GObject.Float(this._factor));
-        this.notify('factor');
-    }
-
-    get orientation() {
-        return this._orientation;
-    }
-
-    set orientation(value) {
-        if (this._orientation == value)
-            return;
-        this._orientation = value;
-        this.notify('orientation');
-    }
-
-    vfunc_pre_paint(context) {
-        let res = super.vfunc_pre_paint(context);
-        let [success, width, height] = this.get_target_size();
-
-        if (success) {
-            // Actually we don't blur horizontally / vertically, but diagonally
-            if (this.orientation != Clutter.Orientation.HORIZONTAL) {
-                this.set_uniform_value('x_step', GObject.Float(1.0 / width));
-                this.set_uniform_value('y_step', GObject.Float(1.0 / height));
-            }
-            else {
-                this.set_uniform_value('x_step', GObject.Float(-1.0 / width));
-                this.set_uniform_value('y_step', GObject.Float(1.0 / height));
-            }
-        }
-        else {
-            this.set_uniform_value('x_step', GObject.Float(0.0));
-            this.set_uniform_value('y_step', GObject.Float(0.0));
-        }
-
-        return res;
-    }
-});
-
-
-var BlurredLightbox = GObject.registerClass(
-class extends Lightbox.Lightbox {
-    _init(container, params) {
-        params.radialEffect = false;
-
-        super._init(container, params);
+        super._init(container, {
+            inhibitEvents: params.inhibitEvents,
+            width: params.width,
+            height: params.height,
+            fadeFactor: 1.0,
+            radialEffect: false,
+        });
 
         if (Clutter.feature_available(Clutter.FeatureFlags.SHADERS_GLSL)) {
-            // Clone the group that contains all of UI on the screen.  This is the
+            // Clone the group that contains all of UI on the screen. This is the
             // chrome, the windows, etc.
-            let uiGroupClone = new Clutter.Clone({ source: Main.uiGroup,
-                                                   clip_to_allocation: true });
-            uiGroupClone.add_effect_with_name('blur1',
-                new BlurEffect({
-                    orientation: Clutter.Orientation.HORIZONTAL,
-                    brightness: this._fadeFactor,
-                    factor: 0.0,
-                }));
-            uiGroupClone.add_effect_with_name('blur2',
-                new BlurEffect({
-                    orientation: Clutter.Orientation.VERTICAL,
-                    brightness: this._fadeFactor,
-                    factor: 0.0,
-                }));
+            this._uiGroup = new Clutter.Clone({ source: Main.uiGroup, clip_to_allocation: true });
+            this._uiGroup.set_background_color(DEFAULT_BACKGROUND_COLOR);
+            this._uiGroup.add_effect_with_name('blur', new Shell.BlurEffect());
+            this.set_child(this._uiGroup);
 
-            this.set_child(uiGroupClone);
-
-            this._uiGroup = uiGroupClone;
-
-            this.set({ opacity: 255 });
+            this.set({ opacity: 0, style_class: 'extension-pomodoro-lightbox-blurred' });
         }
         else {
             this._uiGroup = null;
+
+            this.set({ opacity: 0, style_class: 'extension-pomodoro-lightbox' });
         }
 
-        this.add_style_class_name('extension-pomodoro-lightbox');
+        let themeContext = St.ThemeContext.get_for_stage(global.stage);
+        this._scaleChangedId = themeContext.connect('notify::scale-factor', this._updateEffects.bind(this));
+        this._monitorsChangedId = Main.layoutManager.connect('monitors-changed', this._updateEffects.bind(this));
+
+        this._updateEffects();
+    }
+
+    _updateEffects() {
+        if (this._uiGroup) {
+            const themeContext = St.ThemeContext.get_for_stage(global.stage);
+            let effect = this._uiGroup.get_effect('blur');
+
+            if (effect) {
+                effect.set({
+                    brightness: BLUR_BRIGHTNESS,
+                    sigma: BLUR_SIGMA * themeContext.scale_factor,
+                });
+                effect.queue_repaint();
+            }
+        }
     }
 
     lightOn(fadeInTime) {
-        this.remove_all_transitions();
+        super.lightOn(fadeInTime);
 
-        let easeProps = {
-            duration: fadeInTime || 0,
-            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-        };
+        if (this._uiGroup && !Utils.versionCheck('40.0')) {  // TODO remove compatibility for 3.38
+            let effect = this._uiGroup.get_effect('blur');
+            if (effect) {
+                effect.set({
+                    brightness: BLUR_BRIGHTNESS * 0.99,
+                });
+            }
 
-        let onComplete = () => {
-            this._active = true;
-            this.notify('active');
-        };
-
-        this.show();
-
-        if (this._uiGroup) {
-            this._uiGroup.remove_all_transitions();
-
-            this._uiGroup.ease_property(
-                '@effects.blur1.factor', 1.0, easeProps);
-            this._uiGroup.ease_property(
-                '@effects.blur2.factor', 1.0, Object.assign({ onComplete }, easeProps));
-        } else {
-            this.ease(Object.assign(easeProps, {
-                opacity: 255 * this._fadeFactor,
-                onComplete,
-            }));
+            // HACK: force effect to be repaint itself during fading-in
+            // in theory effect.queue_repaint(); should be enough
+            this._uiGroup.ease_property('@effects.blur.brightness', BLUR_BRIGHTNESS, {
+                duration: fadeInTime || 0,
+            });
         }
     }
 
     lightOff(fadeOutTime) {
-        this.remove_all_transitions();
+        super.lightOff(fadeOutTime);
 
-        this._active = false;
-        this.notify('active');
-
-        let easeProps = {
-            duration: fadeOutTime || 0,
-            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-        };
-
-        let onComplete = () => this.hide();
-
-        if (this._uiGroup) {
-            this._uiGroup.remove_all_transitions();
-
-            this._uiGroup.ease_property(
-                '@effects.blur1.factor', 0.0, easeProps);
-            this._uiGroup.ease_property(
-                '@effects.blur2.factor', 0.0, Object.assign({ onComplete }, easeProps));
-        } else {
-            this.ease(Object.assign(easeProps, { opacity: 0, onComplete }));
+        if (this._uiGroup && !Utils.versionCheck('40.0')) {  // TODO remove compatibility for 3.38
+            let effect = this._uiGroup.get_effect('blur');
+            if (effect) {
+                // HACK: force effect to be repaint itself during fading-out
+                // in theory effect.queue_repaint(); should be enough
+                this._uiGroup.ease_property('@effects.blur.brightness', BLUR_BRIGHTNESS * 0.99, {
+                    duration: fadeOutTime || 0,
+                });
+            }
         }
     }
+
+    /* override parent method */
+    _onDestroy() {
+        if (this._monitorsChangedId) {
+            Main.layoutManager.disconnect(this._monitorsChangedId);
+            delete this._monitorsChangedId;
+        }
+
+        let themeContext = St.ThemeContext.get_for_stage(global.stage);
+        if (this._scaleChangedId) {
+            themeContext.disconnect(this._scaleChangedId);
+            delete this._scaleChangedId;
+        }
+
+        super._onDestroy();
+    }
 });
 
 
@@ -291,55 +183,76 @@ class extends Lightbox.Lightbox {
  * class to have more event signals, different fade in/out times, and different
  * event blocking behavior.
  */
-var ModalDialog = class {
-    constructor() {
-        this.state = State.CLOSED;
+var ModalDialog = GObject.registerClass({
+    Properties: {
+        'state': GObject.ParamSpec.int('state', 'Dialog state', 'state',
+                                       GObject.ParamFlags.READABLE,
+                                       Math.min(...Object.values(State)),
+                                       Math.max(...Object.values(State)),
+                                       State.CLOSED),
+    },
+    Signals: { 'opened': {}, 'opening': {}, 'closed': {}, 'closing': {} },
+}, class PomodoroModalDialog extends St.Widget {
+    _init() {
+        super._init({ style_class: 'extension-pomodoro-dialog',
+                      accessible_role: Atk.Role.DIALOG,
+                      layout_manager: new Clutter.BinLayout(),
+                      reactive: false,
+                      visible: false,
+                      opacity: 0 });
 
-        this._idleMonitor          = Meta.IdleMonitor.get_core();
+        this._state = State.CLOSED;
+        this._hasModal = false;
         this._pushModalDelaySource = 0;
-        this._pushModalWatchId     = 0;
-        this._pushModalSource      = 0;
-        this._pushModalTries       = 0;
-
+        this._pushModalWatchId = 0;
+        this._pushModalSource = 0;
+        this._keyFocusOutId = 0;
         this._monitorConstraint = new Layout.MonitorConstraint();
+        this._monitorConstraint.primary = true;
         this._stageConstraint = new Clutter.BindConstraint({
                                        source: global.stage,
                                        coordinate: Clutter.BindCoordinate.ALL });
+        this.add_constraint(this._stageConstraint);
 
-        this.actor = new St.Widget({ style_class: 'extension-pomodoro-dialog',
-                                     accessible_role: Atk.Role.DIALOG,
-                                     layout_manager: new Clutter.BinLayout(),
-                                     visible: false,
-                                     opacity: 0 });
-        this.actor._delegate = this;
-        this.actor.add_constraint(this._stageConstraint);
-        this.actor.connect('destroy', this._onActorDestroy.bind(this));
+        if (global.backend.get_core_idle_monitor !== undefined) {
+            this._idleMonitor = global.backend.get_core_idle_monitor();
+        }
+        else {
+            this._idleMonitor = Meta.IdleMonitor.get_core();  // TODO: remove along support for gnome-shell 40
+        }
+
+        this.connect('destroy', this._onDestroy.bind(this));
 
         // Modal dialogs are fixed width and grow vertically; set the request
         // mode accordingly so wrapped labels are handled correctly during
         // size requests.
         this._layout = new St.Widget({ layout_manager: new Clutter.BinLayout() });
         this._layout.add_constraint(this._monitorConstraint);
+        this.add_actor(this._layout);
 
-        this.actor.add_actor(this._layout);
-
-        this._lightbox = new BlurredLightbox(this.actor,
-                                             { fadeFactor: FADE_IN_OPACITY,
-                                               inhibitEvents: false });
+        // Lightbox will be a direct child of the ModalDialog
+        this._lightbox = new BlurredLightbox(this,
+                                             { inhibitEvents: false });
         this._lightbox.highlight(this._layout);
 
-        this._grabHelper = new GrabHelper.GrabHelper(this.actor);
-        this._grabHelper.addActor(this._lightbox);
-
-        global.stage.add_actor(this.actor);
-        global.focus_manager.add_group(this.actor);
+        global.stage.add_actor(this);
+        global.focus_manager.add_group(this._lightbox);
     }
 
-    get isOpened() {
-        return this.state == State.OPENED || this.state == State.OPENING;
+    get state() {
+        return this._state;
     }
 
-    _addMessageTray() {
+    _setState(state) {
+        if (this._state == state) {
+            return;
+        }
+
+        this._state = state;
+        this.notify('state');
+    }
+
+    _raiseMessageTray() {
         let messageTray = Main.messageTray;
 
         messageTray.ref();
@@ -348,11 +261,11 @@ var ModalDialog = class {
 
         global.stage.add_child(messageTray);
 
-        messageTray.unref();
         messageTray.bannerBlocked = false;
+        messageTray.unref();
     }
 
-    _removeMessageTray() {
+    _lowerMessageTray() {
         let messageTray = Main.messageTray;
 
         messageTray.ref();
@@ -364,206 +277,137 @@ var ModalDialog = class {
         messageTray.unref();
     }
 
+    _onKeyFocusOut() {
+        let focus = global.stage.key_focus;
+
+        if (focus === null || !this._lightbox.contains(focus)) {
+            this.close(true);
+        }
+    }
+
+    _onOpenComplete() {
+        this._setState(State.OPENED);
+        this.emit('opened');
+    }
+
+    _onIdleMonitorBecameIdle(monitor) {
+        let pushModalTries = 0;
+        let timestamp = global.get_current_time();
+
+        if (this._pushModalWatchId) {
+            this._idleMonitor.remove_watch(this._pushModalWatchId);
+            this._pushModalWatchId = 0;
+        }
+
+        if (this.pushModal(timestamp)) {
+            return GLib.SOURCE_REMOVE;
+        }
+
+        this._pushModalSource = GLib.timeout_add(
+            GLib.PRIORITY_DEFAULT,
+            Math.floor(1000 / PUSH_MODAL_RATE),
+            () => {
+                pushModalTries += 1;
+
+                if (this.pushModal(timestamp)) {
+                    this._pushModalSource = 0;
+                    return GLib.SOURCE_REMOVE;  // success
+                }
+
+                if (pushModalTries > PUSH_MODAL_TIME_LIMIT * PUSH_MODAL_RATE) {
+                    this.close(true);
+                    this._pushModalSource = 0;
+                    return GLib.SOURCE_REMOVE;  // failure
+                }
+
+                return GLib.SOURCE_CONTINUE;
+            });
+        GLib.Source.set_name_by_id(this._pushModalSource,
+                                   '[gnome-pomodoro] this._pushModalSource');
+    }
+
+    // Gradually open the dialog. Try to make it modal once user had chance to see it.
     open(animate) {
         if (this.state == State.OPENED || this.state == State.OPENING) {
             return;
         }
 
-        this.state = State.OPENING;
+        this._pushModalDelaySource = GLib.timeout_add(
+            GLib.PRIORITY_DEFAULT,
+            Math.max(MIN_DISPLAY_TIME - IDLE_TIME_TO_PUSH_MODAL, 0),
+            () => {
+                if (this._pushModalWatchId == 0) {
+                    this._pushModalWatchId = this._idleMonitor.add_idle_watch(IDLE_TIME_TO_PUSH_MODAL,
+                                                                              this._onIdleMonitorBecameIdle.bind(this));
+                }
 
-        if (this._pushModalDelaySource == 0) {
-            this._pushModalDelaySource = GLib.timeout_add(
-                        GLib.PRIORITY_DEFAULT,
-                        Math.max(MIN_DISPLAY_TIME - IDLE_TIME_TO_PUSH_MODAL, 0),
-                        this._onPushModalDelayTimeout.bind(this));
-        }
+                this._pushModalDelaySource = 0;
 
-        // fallback to global.screen.get_current_monitor() for mutter < 3.29
-        this._monitorConstraint.index = typeof(global.display) === 'object' && typeof(global.display.get_current_monitor) !== 'undefined'
-            ? global.display.get_current_monitor() : global.screen.get_current_monitor();
+                return GLib.SOURCE_REMOVE;
+            }
+        );
+        GLib.Source.set_name_by_id(this._pushModalDelaySource,
+                                   '[gnome-pomodoro] this._pushModalDelaySource');
 
-        global.stage.set_child_above_sibling(this.actor, null);
-        this.actor.show();
+        global.stage.set_child_above_sibling(this, null);
 
-        this.actor.remove_all_transitions();
-
-        this._addMessageTray();
+        this.remove_all_transitions();
+        this.show();
+        this._raiseMessageTray();
+        this._setState(State.OPENING);
+        this.emit('opening');
 
         if (animate) {
             this._lightbox.lightOn(FADE_IN_TIME);
-            this.actor.ease({
+
+            this.ease({
                 opacity: 255,
                 duration: FADE_IN_TIME,
                 mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-                onComplete: () => {
-                    if (this.state == State.OPENING) {
-                        this.state = State.OPENED;
-                        this.emit('opened');
-                    }
-                }
+                onComplete: this._onOpenComplete.bind(this),
             });
-            this.emit('opening');
         }
         else {
             this._lightbox.lightOn();
-            this.actor.opacity = 255;
-
-            this.state = State.OPENED;
-
-            this.emit('opening');
-            this.emit('opened');
+            this.opacity = 255;
+            this._onOpenComplete();
         }
     }
 
+    _onCloseComplete() {
+        this.hide();
+        this._lowerMessageTray();
+        this._setState(State.CLOSED);
+        this.emit('closed');
+    }
+
     close(animate) {
-        this._cancelOpenWhenIdle();
-
         if (this.state == State.CLOSED || this.state == State.CLOSING) {
             return;
         }
 
-        this.state = State.CLOSING;
         this.popModal();
+        this._setState(State.CLOSING);
+        this.emit('closing');
 
-        this.actor.remove_all_transitions();
+        this.remove_all_transitions();
 
         if (animate) {
             this._lightbox.lightOff(FADE_OUT_TIME);
-            this.actor.ease({
+            this.ease({
                 opacity: 0,
                 duration: FADE_OUT_TIME,
                 mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-                onComplete: () => {
-                    if (this.state == State.CLOSING) {
-                        this.state = State.CLOSED;
-                        this.actor.hide();
-
-                        this._removeMessageTray();
-
-                        this.emit('closed');
-                    }
-                }
+                onComplete: this._onCloseComplete.bind(this),
             });
-            this.emit('closing');
-        }
-        else {
-            this.actor.opacity = 0;
-            this.actor.hide();
+        } else {
             this._lightbox.lightOff();
-
-            this.state = State.CLOSED;
-
-            this._removeMessageTray();
-
-            this.emit('closing');
-            this.emit('closed');
+            this.opacity = 0;
+            this._onCloseComplete();
         }
     }
 
-    _onPushModalDelayTimeout() {
-        /* Don't become modal and block events just yet,
-         * wait until user becomes idle.
-         */
-        if (this._pushModalWatchId == 0) {
-            this._pushModalWatchId = this._idleMonitor.add_idle_watch(IDLE_TIME_TO_PUSH_MODAL,
-                (monitor) => {
-                    if (this._pushModalWatchId) {
-                        this._idleMonitor.remove_watch(this._pushModalWatchId);
-                        this._pushModalWatchId = 0;
-                    }
-                    this.pushModal();
-                });
-        }
-
-        this._pushModalDelaySource = 0;
-        return GLib.SOURCE_REMOVE;
-    }
-
-    _pushModal() {
-        if (this.state == State.CLOSED || this.state == State.CLOSING) {
-            return false;
-        }
-
-        return this._grabHelper.grab({
-            actor: this._lightbox,
-            focus: this._lightbox,
-            onUngrab: this._onUngrab.bind(this)
-        });
-    }
-
-    _onPushModalTimeout() {
-        if (this.state == State.CLOSED || this.state == State.CLOSING) {
-            this._pushModalSource = 0;
-            return GLib.SOURCE_REMOVE;
-        }
-
-        this._pushModalTries += 1;
-
-        if (this._pushModal()) {
-            this._pushModalSource = 0;
-            return GLib.SOURCE_REMOVE; /* dialog finally opened */
-        }
-
-        if (this._pushModalTries > PUSH_MODAL_TIME_LIMIT * PUSH_MODAL_RATE) {
-            this.close(true);
-            this._pushModalSource = 0;
-            return GLib.SOURCE_REMOVE; /* dialog can't become modal */
-        }
-
-        return GLib.SOURCE_CONTINUE;
-    }
-
-    pushModal() {
-        if (this.state == State.CLOSED || this.state == State.CLOSING) {
-            return;
-        }
-
-        this._disconnectSignals();
-
-        this._lightbox.reactive = true;
-
-        // this._grabHelper.ignoreRelease();
-
-        /* delay pushModal to ignore current events */
-        GLib.idle_add(
-            GLib.PRIORITY_DEFAULT,
-            () => {
-                this._pushModalTries = 1;
-
-                if (this._pushModal()) {
-                    /* dialog became modal */
-                }
-                else {
-                    this._pushModalSource = GLib.timeout_add(GLib.PRIORITY_DEFAULT,
-                                                             Math.floor(1000 / PUSH_MODAL_RATE),
-                                                             this._onPushModalTimeout.bind(this));
-                }
-
-                return GLib.SOURCE_REMOVE;
-            });
-    }
-
-    /**
-     * Drop modal status without closing the dialog; this makes the
-     * dialog insensitive as well, so it needs to be followed shortly
-     * by either a close() or a pushModal()
-     */
-    popModal() {
-        try {
-            if (this._grabHelper.isActorGrabbed(this._lightbox)) {
-                this._grabHelper.ungrab({
-                    actor: this._lightbox
-                });
-            }
-        }
-        catch (error) {
-            Utils.logWarning(error.message);
-        }
-
-        this._disconnectSignals();
-    }
-
-    _disconnectSignals() {
+    _disconnectPushModalSignals() {
         if (this._pushModalDelaySource) {
             GLib.source_remove(this._pushModalDelaySource);
             this._pushModalDelaySource = 0;
@@ -580,32 +424,72 @@ var ModalDialog = class {
         }
     }
 
-    _onUngrab() {
-        this.close(true);
-    }
+    // Drop modal status without closing the dialog; this makes the
+    // dialog insensitive as well, so it needs to be followed shortly
+    // by either a close() or a pushModal()
+    popModal(timestamp) {
+        this._disconnectPushModalSignals();
 
-    _onActorDestroy() {
-        if (this._destroyed)
+        if (this._keyFocusOutId) {
+            this._lightbox.disconnect(this._keyFocusOutId);
+            this._keyFocusOutId = 0;
+        }
+
+        if (!this._hasModal) {
             return;
-        this._destroyed = true;
+        }
 
-        this.close(false);
-
-        this.actor._delegate = null;
-
-        this.emit('destroy');
+        Main.popModal(this, timestamp);
+        this._hasModal = false;
+        this._lightbox.reactive = false;
     }
 
-    destroy() {
-        this.actor.destroy();
+    pushModal(timestamp) {
+        if (this._hasModal) {
+            return true;
+        }
+
+        if (this.state == State.CLOSED || this.state == State.CLOSING) {
+            return false;
+        }
+
+        let params = { actionMode: Shell.ActionMode.SYSTEM_MODAL };
+        if (timestamp) {
+            params['timestamp'] = timestamp;
+        }
+
+        if (!Main.pushModal(this, params)) {
+            return;
+        }
+
+        this._disconnectPushModalSignals();
+        this._hasModal = true;
+        this._lightbox.reactive = true;
+
+        global.stage.set_key_focus(this._lightbox);
+
+        if (!this._keyFocusOutId) {
+            this._keyFocusOutId = this._lightbox.connect('key-focus-out', this._onKeyFocusOut.bind(this));
+        }
+
+        Main.layoutManager.emit('system-modal-opened');
     }
-};
-Signals.addSignalMethods(ModalDialog.prototype);
+
+    _onDestroy() {
+        this.popModal();
+
+        if (this._lightbox) {
+            this._lightbox.destroy();
+            this._lightbox = null;
+        }
+    }
+});
 
 
-var PomodoroEndDialog = class extends ModalDialog {
-    constructor(timer) {
-        super();
+var PomodoroEndDialog = GObject.registerClass(
+class PomodoroEndDialog extends ModalDialog {
+    _init(timer) {
+        super._init();
 
         this.timer = timer;
         this.description = _("It's time to take a break");
@@ -613,10 +497,11 @@ var PomodoroEndDialog = class extends ModalDialog {
         this._openWhenIdleWatchId        = 0;
         this._closeWhenActiveDelaySource = 0;
         this._closeWhenActiveIdleWatchId = 0;
-        this._actorMappedId              = 0;
+        this._mappedId                   = 0;
         this._timerUpdateId              = 0;
         this._eventId                    = 0;
         this._styleChangedId             = 0;
+        this._closingId                  = 0;
 
         this._minutesLabel = new St.Label({
             x_expand: true,
@@ -649,14 +534,12 @@ var PomodoroEndDialog = class extends ModalDialog {
         box.add_actor(this._descriptionLabel);
         this._layout.add_actor(box);
 
-        this._actorMappedId = this.actor.connect('notify::mapped', this._onActorMappedChanged.bind(this));
-
-        this.connect('closing', this._onClosing.bind(this));
-        this.connect('destroy', this._onDestroy.bind(this));
+        this._mappedId = this.connect('notify::mapped', this._onMappedChanged.bind(this));
+        this._closingId = this.connect('closing', this._onClosing.bind(this));
     }
 
-    _onActorMappedChanged(actor) {
-        if (actor.mapped) {
+    _onMappedChanged() {
+        if (this.mapped) {
             if (!this._styleChangedId) {
                 this._styleChangedId = this._secondsLabel.connect('style-changed', this._onStyleChanged.bind(this));
                 this._onStyleChanged(this._secondsLabel);
@@ -667,6 +550,10 @@ var PomodoroEndDialog = class extends ModalDialog {
             }
         }
         else {
+            if (this._styleChangedId) {
+                this._secondsLabel.disconnect(this._styleChangedId);
+                this._styleChangedId = 0;
+            }
             if (this._timerUpdateId) {
                 this.timer.disconnect(this._timerUpdateId);
                 this._timerUpdateId = 0;
@@ -690,7 +577,7 @@ var PomodoroEndDialog = class extends ModalDialog {
             let minutes   = Math.floor(remaining / 60);
             let seconds   = Math.floor(remaining % 60);
 
-            /* method may be called while label actor got destroyed */
+            // method may be called while label actor got destroyed
             if (this._minutesLabel.clutter_text) {
                 this._minutesLabel.clutter_text.set_text('%d'.format(minutes));
             }
@@ -700,9 +587,17 @@ var PomodoroEndDialog = class extends ModalDialog {
         }
     }
 
-    _onClosing() {
-        this._cancelCloseWhenActive();
-        this._cancelOpenWhenIdle();
+    // disconecct signals that are no longer neeeded after dialog closes
+    _disconnectSignals() {
+        if (this._openWhenIdleWatchId) {
+            this._idleMonitor.remove_watch(this._openWhenIdleWatchId);
+            this._openWhenIdleWatchId = 0;
+        }
+
+        if (this._eventId) {
+            this._lightbox.disconnect(this._eventId);
+            this._eventId = 0;
+        }
 
         if (this._closeWhenActiveDelaySource) {
             GLib.source_remove(this._closeWhenActiveDelaySource);
@@ -725,19 +620,10 @@ var PomodoroEndDialog = class extends ModalDialog {
         }
     }
 
-    _onDestroy() {
-        this._onClosing();
-
-        if (this._actorMappedId) {
-            this.actor.disconnect(this._actorMappedId);
-            this._actorMappedId = 0;
-        }
-    }
-
     _onEvent(actor, event) {
         let x, y, dx, dy, distance;
 
-        if (!event.get_device ()) {
+        if (!event.get_device()) {
             return Clutter.EVENT_STOP;
         }
 
@@ -769,8 +655,24 @@ var PomodoroEndDialog = class extends ModalDialog {
             case Clutter.EventType.KEY_PRESS:
                 switch (event.get_key_symbol())
                 {
+                    case Clutter.KEY_AudioCycleTrack:
+                    case Clutter.KEY_AudioForward:
                     case Clutter.KEY_AudioLowerVolume:
+                    case Clutter.KEY_AudioNext:
+                    case Clutter.KEY_AudioPause:
+                    case Clutter.KEY_AudioPlay:
+                    case Clutter.KEY_AudioPrev:
                     case Clutter.KEY_AudioRaiseVolume:
+                    case Clutter.KEY_AudioRandomPlay:
+                    case Clutter.KEY_AudioRecord:
+                    case Clutter.KEY_AudioRepeat:
+                    case Clutter.KEY_AudioRewind:
+                    case Clutter.KEY_AudioStop:
+                    case Clutter.KEY_AudioMicMute:
+                    case Clutter.KEY_AudioMute:
+                    case Clutter.KEY_MonBrightnessDown:
+                    case Clutter.KEY_MonBrightnessUp:
+                    case Clutter.KEY_Display:
                         return Clutter.EVENT_PROPAGATE;
 
                     default:
@@ -783,48 +685,72 @@ var PomodoroEndDialog = class extends ModalDialog {
             case Clutter.EventType.BUTTON_PRESS:
             case Clutter.EventType.TOUCH_BEGIN:
                 this.close(true);
-
                 break;
         }
 
         return Clutter.EVENT_STOP;
     }
 
-    /**
-     * Open the dialog and setup closing when user becomes active.
-     */
+    _onClosing() {
+        this._disconnectSignals();
+    }
+
+    _onDestroy() {
+        this._disconnectSignals()
+
+        if (this._closingId) {
+            this.disconnect(this._closingId);
+            this._closingId = 0;
+        }
+
+        if (this._actorMappedId) {
+            this.disconnect(this._actorMappedId);
+            this._actorMappedId = 0;
+        }
+
+        super._onDestroy();
+    }
+
+    _closeWhenActive() {
+        if (this.state == State.CLOSED || this.state == State.CLOSING) {
+            return;
+        }
+
+        if (this._eventId == 0) {
+            this._eventX = -1;
+            this._eventY = -1;
+            this._eventId = this._lightbox.connect('event', this._onEvent.bind(this));
+        }
+    }
+
+    // Open the dialog. Wait until user had chance of seeing the dialog
+    // and schedule to close it once user becomes active.
     open(animate) {
         super.open(animate);
 
-        /* Wait until user has a chance of seeing the dialog */
         if (this._closeWhenActiveDelaySource == 0) {
             this._closeWhenActiveDelaySource = GLib.timeout_add(
                 GLib.PRIORITY_DEFAULT,
                 MIN_DISPLAY_TIME,
                 () => {
                     if (this._idleMonitor.get_idletime() < IDLE_TIME_TO_CLOSE) {
-                        /* Wait until user becomes slightly idle */
-                        this._closeWhenActiveIdleWatchId =
-                                this._idleMonitor.add_idle_watch(IDLE_TIME_TO_CLOSE,
-                                                                 this.closeWhenActive.bind(this));
+                        this._closeWhenActiveIdleWatchId = this._idleMonitor.add_idle_watch(IDLE_TIME_TO_CLOSE,
+                            (monitor) => this._closeWhenActive()
+                        );
                     }
                     else {
-                        this.closeWhenActive();
+                        this._closeWhenActive();
                     }
 
                     this._closeWhenActiveDelaySource = 0;
                     return GLib.SOURCE_REMOVE;
                 });
+            GLib.Source.set_name_by_id(this._closeWhenActiveDelaySource,
+                                       '[gnome-pomodoro] this._closeWhenActiveDelaySource');
         }
     }
 
-    _cancelOpenWhenIdle() {
-        if (this._openWhenIdleWatchId) {
-            this._idleMonitor.remove_watch(this._openWhenIdleWatchId);
-            this._openWhenIdleWatchId = 0;
-        }
-    }
-
+    // Schedule dialog to open when idle
     openWhenIdle() {
         if (this.state == State.OPEN || this.state == State.OPENING) {
             return;
@@ -832,12 +758,12 @@ var PomodoroEndDialog = class extends ModalDialog {
 
         if (this._openWhenIdleWatchId == 0) {
             this._openWhenIdleWatchId = this._idleMonitor.add_idle_watch(IDLE_TIME_TO_OPEN,
-                () => {
+                (monitor) => {
                     let info = Utils.getFocusedWindowInfo();
 
                     if (info.isPlayer && info.isFullscreen)
                     {
-                        /* dont reopen if playing a video */
+                        // dont reopen if playing a video
                         return;
                     }
 
@@ -852,26 +778,6 @@ var PomodoroEndDialog = class extends ModalDialog {
         }
     }
 
-    _cancelCloseWhenActive() {
-        if (this._eventId) {
-            this._lightbox.disconnect(this._eventId);
-            this._eventId = 0;
-        }
-    }
-
-    // TODO: should be private
-    closeWhenActive() {
-        if (this.state == State.CLOSED || this.state == State.CLOSING) {
-            return;
-        }
-
-        if (this._eventId == 0) {
-            this._eventX = -1;
-            this._eventY = -1;
-            this._eventId = this._lightbox.connect('event', this._onEvent.bind(this));
-        }
-    }
-
     setDescription(text) {
         this.description = text;
 
@@ -879,4 +785,4 @@ var PomodoroEndDialog = class extends ModalDialog {
             this._descriptionLabel.clutter_text.set_text(this.description);
         }
     }
-};
+});
