@@ -24,6 +24,7 @@ const Signals = imports.signals;
 const { Clutter, Gio, GLib, GObject, Gtk, Meta, Pango, Shell, St } = imports.gi;
 
 const Extension = imports.misc.extensionUtils.getCurrentExtension();
+const BoxPointer = imports.ui.boxpointer;
 const Main = imports.ui.main;
 const MessageTray = imports.ui.messageTray;
 const PanelMenu = imports.ui.panelMenu;
@@ -60,6 +61,7 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
         this._isPaused = null;
         this._timerState = null;
         this._timerUpdateId = 0;
+        this._icons = {};
 
         this.actor.add_style_class_name('extension-pomodoro-indicator-menu');
 
@@ -70,35 +72,62 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
         this._populate();
     }
 
+    _loadIcon(iconName) {
+        let icon = this._icons[iconName];
+
+        if (!icon) {
+            let iconFile = Gio.File.new_for_uri('%s/icons/hicolor/scalable/actions/%s.svg'.format(Extension.dir.get_uri(), iconName));
+            icon = new Gio.FileIcon({ file: iconFile });
+
+            this._icons[iconName] = icon;
+        }
+
+        return icon;
+    }
+
     _createActionButton(iconName, accessibleName) {
         let button = new St.Button({ reactive: true,
                                      can_focus: true,
                                      track_hover: true,
                                      accessible_name: accessibleName,
-                                     style_class: 'system-menu-action extension-pomodoro-indicator-menu-action' });
-        button.child = new St.Icon({ icon_name: iconName });
+                                     style_class: 'extension-pomodoro-indicator-menu-action' });
+        button.child = new St.Icon({ gicon: this._loadIcon(iconName), style_class: 'popup-menu-icon' });
+
         return button;
     }
 
     _onTimerClicked() {
-        this.close();
+        if (this._isPaused) {
+            this.itemActivated(BoxPointer.PopupAnimation.NONE);
+            this.indicator.timer.resume();
+            return;
+        }
 
-        if (Extension.extension && Extension.extension.dialog) {
+        if (this._timerState != Timer.State.POMODORO && Extension.extension && Extension.extension.dialog) {
+            this.itemActivated(BoxPointer.PopupAnimation.NONE);
             Extension.extension.dialog.open(true);
             Extension.extension.dialog.pushModal();
+            return;
         }
     }
 
     _onStartClicked() {
+        this.itemActivated(BoxPointer.PopupAnimation.NONE);
         this.indicator.timer.start();
-
-        this.close();
     }
 
     _onStopClicked() {
-        this.indicator.timer.stop();
+        let id = GLib.idle_add(GLib.PRIORITY_DEFAULT, () => {
+            this.indicator.timer.stop();
+            return GLib.SOURCE_REMOVE;
+        });
+        GLib.Source.set_name_by_id(id, '[gnome-pomodoro] this.indicator.timer.stop()');
 
-        this.close();
+        // Closing menu leads to GrabHelper complaing about accessing deallocated St.Button,
+        // while doing savedFocus.grab_key_focus().
+        // As a walkaround we call timer.stop() with delay. Seems that these calls interfere
+        // with each other.
+        this.itemActivated(BoxPointer.PopupAnimation.NONE);
     }
 
     _onPauseClicked() {
@@ -106,9 +135,8 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
             this.indicator.timer.pause();
         }
         else {
+            this.itemActivated(BoxPointer.PopupAnimation.NONE);
             this.indicator.timer.resume();
-
-            this.close();
         }
     }
 
@@ -120,8 +148,7 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
         toggleItem.label.y_align = Clutter.ActorAlign.CENTER;
         this.addMenuItem(toggleItem);
 
-        let startAction = this._createActionButton('media-playback-start-symbolic', _("Start Timer"));
-        startAction.add_style_class_name('extension-pomodoro-indicator-menu-action-border');
+        let startAction = this._createActionButton('gnome-pomodoro-start-symbolic', _("Start Timer"));
         startAction.connect('clicked', this._onStartClicked.bind(this));
         toggleItem.add_child(startAction);
 
@@ -146,11 +173,11 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
                                       x_expand: true });
         timerItem.add_child(hbox);
 
-        let pauseAction = this._createActionButton('media-playback-pause-symbolic', _("Pause Timer"));
+        let pauseAction = this._createActionButton('gnome-pomodoro-pause-symbolic', _("Pause Timer"));
         pauseAction.connect('clicked', this._onPauseClicked.bind(this));
         hbox.add_actor(pauseAction);
 
-        let stopAction = this._createActionButton('media-playback-stop-symbolic', _("Stop Timer"));
+        let stopAction = this._createActionButton('gnome-pomodoro-stop-symbolic', _("Stop Timer"));
         stopAction.connect('clicked', this._onStopClicked.bind(this));
         hbox.add_actor(stopAction);
 
@@ -216,10 +243,10 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
             this._toggleMenuItem.visible = !isRunning;
             this._timerMenuItem.visible = isRunning;
 
-            this._timerLabelButton.reactive = isRunning && !isPaused && timerState != Timer.State.POMODORO;
-            this.pauseAction.child.icon_name = isPaused
-                                               ? 'media-playback-start-symbolic'
-                                               : 'media-playback-pause-symbolic';
+            this._timerLabelButton.reactive = isPaused || isRunning && timerState != Timer.State.POMODORO;
+            this.pauseAction.child.gicon = isPaused
+                                               ? this._loadIcon('gnome-pomodoro-start-symbolic')
+                                               : this._loadIcon('gnome-pomodoro-pause-symbolic');
             this.pauseAction.accessible_name = isPaused
                                                ? _("Resume Timer")
                                                : _("Pause Timer");
@@ -257,12 +284,14 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
     _activateStats() {
         let timestamp = global.get_current_time();
 
+        Main.overview.hide();
         this.indicator.timer.showMainWindow('stats', timestamp);
     }
 
     _activatePreferences() {
         let timestamp = global.get_current_time();
 
+        Main.overview.hide();
         this.indicator.timer.showPreferences(timestamp);
     }
 
@@ -271,9 +300,19 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
     }
 
     _activateState(stateName) {
+        this.itemActivated(BoxPointer.PopupAnimation.NONE);
         this.indicator.timer.setState(stateName);
     }
 
+    close(animate) {
+        if (this._timerUpdateId) {
+            this.indicator.timer.disconnect(this._timerUpdateId);
+            this._timerUpdateId = 0;
+        }
+
+        super.close(animate);
+    }
+
     destroy() {
         if (this._timerUpdateId) {
             this.indicator.timer.disconnect(this._timerUpdateId);
@@ -288,6 +327,7 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
         this.indicator = null;
         this.timerLabel = null;
         this.pauseAction = null;
+        this._icons = null;
 
         super.destroy();
     }
@@ -298,8 +338,6 @@ var TextIndicator = class {
     constructor(timer) {
         this._initialized     = false;
         this._state           = Timer.State.NULL;
-        this._minHPadding     = 0;
-        this._natHPadding     = 0;
         this._digitWidth      = 0;
         this._charWidth       = 0;
         this._onTimerUpdateId = 0;
@@ -342,13 +380,12 @@ var TextIndicator = class {
     }
 
     _onStyleChanged(actor) {
-        let themeNode = actor.get_theme_node();
-        let font      = themeNode.get_font();
-        let context   = actor.get_pango_context();
-        let metrics   = context.get_metrics(font, context.get_language());
+        let themeNode    = actor.get_theme_node();
+        let themeContext = St.ThemeContext.get_for_stage(global.stage);
+        let font         = themeNode.get_font();
+        let context      = actor.get_pango_context();
+        let metrics      = context.get_metrics(font, context.get_language());
 
-        this._minHPadding = themeNode.get_length('-minimum-hpadding');
-        this._natHPadding = themeNode.get_length('-natural-hpadding');
         this._digitWidth  = metrics.get_approximate_digit_width() / Pango.SCALE;
         this._charWidth   = metrics.get_approximate_char_width() / Pango.SCALE;
     }
@@ -446,10 +483,6 @@ var IconIndicator = class {
     constructor(timer) {
         this._state           = Timer.State.NULL;
         this._progress        = 0.0;
-        this._minHPadding     = 0;
-        this._natHPadding     = 0;
-        this._minVPadding     = 0;
-        this._natVPadding     = 0;
         this._primaryColor    = null;
         this._secondaryColor  = null;
         this._timerUpdateId = 0;
@@ -477,7 +510,10 @@ var IconIndicator = class {
 
     _onIconStyleChanged(actor) {
         let themeNode = actor.get_theme_node();
-        let size = Math.ceil(themeNode.get_length('icon-size'));
+        let [found, size] = themeNode.lookup_length('icon-size', false);
+
+        if (!found)
+            return;
 
         [actor.min_width, actor.natural_width] = themeNode.adjust_preferred_width(size, size);
         [actor.min_height, actor.natural_height] = themeNode.adjust_preferred_height(size, size);
@@ -541,11 +577,6 @@ var IconIndicator = class {
     _onStyleChanged(actor) {
         let themeNode = actor.get_theme_node();
 
-        this._minHPadding = themeNode.get_length('-minimum-hpadding');
-        this._natHPadding = themeNode.get_length('-natural-hpadding');
-        this._minVPadding = themeNode.get_length('-minimum-vpadding');
-        this._natVPadding = themeNode.get_length('-natural-vpadding');
-
         let color = themeNode.get_foreground_color();
         this._primaryColor = color;
         this._secondaryColor = new Clutter.Color({
@@ -600,41 +631,76 @@ class PomodoroIndicator extends PanelMenu.Button {
 
         this.add_style_class_name('extension-pomodoro-indicator');
 
-        this._arrow = PopupMenu.arrowIcon(St.Side.BOTTOM);
-        this._blinking = false;
-        this._blinkingGroup = new Utils.TransitionGroup();
-
         this._hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
         this._hbox.pack_start = true;
         this._hbox.set_x_align(Clutter.ActorAlign.CENTER);
         this._hbox.set_y_align(Clutter.ActorAlign.CENTER);
-        this._hbox.add_child(this._arrow);
         this.add_child(this._hbox);
 
         this.setMenu(new IndicatorMenu(this));
         this.setType(type);
 
+        this._blinking = false;
+        this._blinkingGroup = new Utils.TransitionGroup();
         this._blinkingGroup.addActor(this._hbox);
         this._blinkingGroup.addActor(this.menu.timerLabel);
         this._blinkingGroup.addActor(this.menu.pauseAction.child);
-        this._onBlinked();
 
-        this._timerPausedId = this.timer.connect('paused', this._onTimerPaused.bind(this));
-        this._timerResumedId = this.timer.connect('resumed', this._onTimerResumed.bind(this));
+        this._mappedId = this.connect('notify::mapped', this._onMappedChanged.bind(this));
 
-        let destroyId = this.connect('destroy', () => {
-            this.timer.disconnect(this._timerPausedId);
-            this.timer.disconnect(this._timerResumedId);
-            this.timer = null;
-            this._blinkingGroup.destroy();
+        this.connect('destroy', () => {
+            if (this._timerPausedId) {
+                this.timer.disconnect(this._timerPausedId);
+                this._timerPausedId = 0;
+            }
+
+            if (this._timerResumedId) {
+                this.timer.disconnect(this._timerResumedId);
+                this._timerResumedId = 0;
+            }
+
+            if (this._blinkingGroup) {
+                this._blinkingGroup.destroy();
+                this._blinkingGroup = null;
+            }
 
             if (this.icon) {
                 this.icon.destroy();
                 this.icon = null;
             }
 
-            this.disconnect(destroyId);
-        })
+            if (this._mappedId) {
+                this.disconnect(this._mappedId);
+                this._mappedId = 0;
+            }
+
+            this.timer = null;
+        });
+    }
+
+    _onMappedChanged() {
+        if (this.mapped) {
+            if (!this._timerPausedId) {
+                this._timerPausedId = this.timer.connect('paused', this._onTimerPaused.bind(this));
+            }
+
+            if (!this._timerResumedId) {
+                this._timerResumedId = this.timer.connect('resumed', this._onTimerResumed.bind(this));
+            }
+        }
+        else {
+            if (this._timerPausedId) {
+                this.timer.disconnect(this._timerPausedId);
+                this._timerPausedId = 0;
+            }
+
+            if (this._timerResumedId) {
+                this.timer.disconnect(this._timerResumedId);
+                this._timerResumedId = 0;
+            }
+        }
+
+        this._onBlinked();
     }
 
     setType(type) {
@@ -657,30 +723,48 @@ class PomodoroIndicator extends PanelMenu.Button {
                 break;
         }
 
-        this.widget.actor.bind_property('opacity',
-                                        this._arrow,
-                                        'opacity',
-                                        GObject.BindingFlags.SYNC_CREATE);
-
         this._hbox.add_child(this.widget.actor);
     }
 
     _onBlinked() {
         this._blinking = false;
 
+        if (!this.mapped) {
+            this._blinkingGroup.removeAllTransitions();
+            this._blinkingGroup.setProperty('opacity', 255);
+        }
+
         if (this.timer.isPaused()) {
             this._blink();
         }
     }
 
     _blink() {
+        if (!this.mapped) {
+            return;
+        }
+
         if (!this._blinking) {
+            let ignoreSignals = false;
             let fadeIn = () => {
                 if (this._blinking) {
                     this._blinkingGroup.easeProperty('opacity', FADE_IN_OPACITY * 255, {
                         duration: 1750,
                         mode: Clutter.AnimationMode.EASE_IN_OUT_CUBIC,
-                        onComplete: fadeOut,
+                        onComplete: () => {
+                            if (!this.mapped) {
+                                return;
+                            }
+
+                            if (!ignoreSignals) {
+                                ignoreSignals = true;
+                                fadeOut()
+                                ignoreSignals = false;
+                            }
+                            else {
+                                // stop recursion
+                            }
+                        },
                     });
                 }
                 else {
@@ -692,7 +776,16 @@ class PomodoroIndicator extends PanelMenu.Button {
                     this._blinkingGroup.easeProperty('opacity', FADE_OUT_OPACITY * 255, {
                         duration: 1750,
                         mode: Clutter.AnimationMode.EASE_IN_OUT_CUBIC,
-                        onComplete: fadeIn,
+                        onComplete: () => {
+                            if (!ignoreSignals) {
+                                ignoreSignals = true;
+                                fadeIn();
+                                ignoreSignals = false;
+                            }
+                            else {
+                                // stop recursion
+                            }
+                        }
                     });
                 }
                 else {
